@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Edge resistance for move/resize operations */
 
 /* 
@@ -44,7 +46,6 @@ struct ResistanceDataForAnEdge
   GSourceFunc  timeout_func;
   MetaWindow  *window;
   int          keyboard_buildup;
-  gboolean     allow_past_screen_edge;
 };
 typedef struct ResistanceDataForAnEdge ResistanceDataForAnEdge;
 
@@ -61,6 +62,9 @@ struct MetaEdgeResistanceData
   ResistanceDataForAnEdge bottom_data;
 };
 
+/* !WARNING!: this function can return invalid indices (namely, either -1 or
+ * edges->len); this is by design, but you need to remember this.
+ */
 static int
 find_index_of_edge_near_position (const GArray *edges,
                                   int           position,
@@ -329,6 +333,7 @@ apply_edge_resistance (MetaWindow                *window,
                        gboolean                   keyboard_op)
 {
   int i, begin, end;
+  int last_edge;
   gboolean increasing = new_pos > old_pos;
   int      increment = increasing ? 1 : -1;
 
@@ -365,6 +370,13 @@ apply_edge_resistance (MetaWindow                *window,
   begin = find_index_of_edge_near_position (edges, old_pos,  increasing, xdir);
   end   = find_index_of_edge_near_position (edges, new_pos, !increasing, xdir);
 
+  /* begin and end can be outside the array index, if the window is partially
+   * off the screen
+   */
+  last_edge = edges->len - 1;
+  begin = CLAMP (begin, 0, last_edge);
+  end   = CLAMP (end,   0, last_edge);
+
   /* Loop over all these edges we're moving past/to. */
   i = begin;
   while ((increasing  && i <= end) ||
@@ -397,18 +409,6 @@ apply_edge_resistance (MetaWindow                *window,
         {
           int threshold;
 
-          /* INFINITE RESISTANCE for screen edges under certain cases; If
-           * the edge is relevant and we're moving towards it and it's a
-           * screen edge and infinite resistance has been requested for
-           * this particular grab op then don't allow movement past it.
-           */
-          if (edge->edge_type == META_EDGE_SCREEN &&
-              !resistance_data->allow_past_screen_edge &&
-              movement_towards_edge (edge->side_type, increment))
-            {
-              return compare;
-            }
-
           /* TIMEOUT RESISTANCE: If the edge is relevant and we're moving
            * towards it, then we may want to have some kind of time delay
            * before the user can move past this edge.
@@ -541,7 +541,8 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
                                     MetaRectangle       *new_outer,
                                     GSourceFunc          timeout_func,
                                     gboolean             auto_snap,
-                                    gboolean             keyboard_op)
+                                    gboolean             keyboard_op,
+                                    gboolean             is_resize)
 {
   MetaEdgeResistanceData *edge_data;
   MetaRectangle           modified_rect;
@@ -588,47 +589,68 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
     }
   else
     {
-      /* Now, apply the normal edge resistance */
-      new_left   = apply_edge_resistance (window,
-                                          BOX_LEFT (*old_outer),
-                                          BOX_LEFT (*new_outer),
-                                          old_outer,
-                                          new_outer,
-                                          edge_data->left_edges,
-                                          &edge_data->left_data,
-                                          timeout_func,
-                                          TRUE,
-                                          keyboard_op);
-      new_right  = apply_edge_resistance (window,
-                                          BOX_RIGHT (*old_outer),
-                                          BOX_RIGHT (*new_outer),
-                                          old_outer,
-                                          new_outer,
-                                          edge_data->right_edges,
-                                          &edge_data->right_data,
-                                          timeout_func,
-                                          TRUE,
-                                          keyboard_op);
-      new_top    = apply_edge_resistance (window,
-                                          BOX_TOP (*old_outer),
-                                          BOX_TOP (*new_outer),
-                                          old_outer,
-                                          new_outer,
-                                          edge_data->top_edges,
-                                          &edge_data->top_data,
-                                          timeout_func,
-                                          FALSE,
-                                          keyboard_op);
-      new_bottom = apply_edge_resistance (window,
-                                          BOX_BOTTOM (*old_outer),
-                                          BOX_BOTTOM (*new_outer),
-                                          old_outer,
-                                          new_outer,
-                                          edge_data->bottom_edges,
-                                          &edge_data->bottom_data,
-                                          timeout_func,
-                                          FALSE,
-                                          keyboard_op);
+      /* Disable edge resistance for resizes when windows have size
+       * increment hints; see #346782.  For all other cases, apply
+       * them.
+       */
+      if (!is_resize || window->size_hints.width_inc == 1)
+        {
+          /* Now, apply the normal horizontal edge resistance */
+          new_left   = apply_edge_resistance (window,
+                                              BOX_LEFT (*old_outer),
+                                              BOX_LEFT (*new_outer),
+                                              old_outer,
+                                              new_outer,
+                                              edge_data->left_edges,
+                                              &edge_data->left_data,
+                                              timeout_func,
+                                              TRUE,
+                                              keyboard_op);
+          new_right  = apply_edge_resistance (window,
+                                              BOX_RIGHT (*old_outer),
+                                              BOX_RIGHT (*new_outer),
+                                              old_outer,
+                                              new_outer,
+                                              edge_data->right_edges,
+                                              &edge_data->right_data,
+                                              timeout_func,
+                                              TRUE,
+                                              keyboard_op);
+        }
+      else
+        {
+          new_left  = new_outer->x;
+          new_right = new_outer->x + new_outer->width;
+        }
+      /* Same for vertical resizes... */
+      if (!is_resize || window->size_hints.height_inc == 1)
+        {
+          new_top    = apply_edge_resistance (window,
+                                              BOX_TOP (*old_outer),
+                                              BOX_TOP (*new_outer),
+                                              old_outer,
+                                              new_outer,
+                                              edge_data->top_edges,
+                                              &edge_data->top_data,
+                                              timeout_func,
+                                              FALSE,
+                                              keyboard_op);
+          new_bottom = apply_edge_resistance (window,
+                                              BOX_BOTTOM (*old_outer),
+                                              BOX_BOTTOM (*new_outer),
+                                              old_outer,
+                                              new_outer,
+                                              edge_data->bottom_edges,
+                                              &edge_data->bottom_data,
+                                              timeout_func,
+                                              FALSE,
+                                              keyboard_op);
+        }
+      else
+        {
+          new_top    = new_outer->y;
+          new_bottom = new_outer->y + new_outer->height;
+        }
     }
 
   /* Determine whether anything changed, and save the changes */
@@ -646,10 +668,13 @@ meta_display_cleanup_edges (MetaDisplay *display)
 {
   guint i,j;
   MetaEdgeResistanceData *edge_data = display->grab_edge_resistance_data;
+  GHashTable *edges_to_be_freed;
 
   g_assert (edge_data != NULL);
 
   /* We first need to clean out any window edges */
+  edges_to_be_freed = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                                             g_free, NULL);
   for (i = 0; i < 4; i++)
     {
       GArray *tmp = NULL;
@@ -681,15 +706,29 @@ meta_display_cleanup_edges (MetaDisplay *display)
           MetaEdge *edge = g_array_index (tmp, MetaEdge*, j);
           if (edge->edge_type == META_EDGE_WINDOW &&
               edge->side_type == dir)
-            g_free (edge);
+            {
+              /* The same edge will appear in two arrays, and we can't free
+               * it yet we still need to compare edge->side_type for the other
+               * array that it is in.  So store it in a hash table for later
+               * freeing.  Could also do this in a simple linked list.
+               */
+              g_hash_table_insert (edges_to_be_freed, edge, edge);
+            }
         }
     }
 
+  /* Now free all the window edges (the key destroy function is g_free) */
+  g_hash_table_destroy (edges_to_be_freed);
+
   /* Now free the arrays and data */
   g_array_free (edge_data->left_edges, TRUE);
   g_array_free (edge_data->right_edges, TRUE);
   g_array_free (edge_data->top_edges, TRUE);
   g_array_free (edge_data->bottom_edges, TRUE);
+  edge_data->left_edges = NULL;
+  edge_data->right_edges = NULL;
+  edge_data->top_edges = NULL;
+  edge_data->bottom_edges = NULL;
 
   /* Cleanup the timeouts */
   if (edge_data->left_data.timeout_setup   &&
@@ -729,6 +768,31 @@ cache_edges (MetaDisplay *display,
   int num_left, num_right, num_top, num_bottom;
   int i;
 
+  /*
+   * 0th: Print debugging information to the log about the edges
+   */
+#ifdef WITH_VERBOSE_MODE
+  if (meta_is_verbose())
+    {
+      int max_edges = MAX (MAX( g_list_length (window_edges), 
+                                g_list_length (xinerama_edges)),
+                           g_list_length (screen_edges));
+      char big_buffer[(EDGE_LENGTH+2)*max_edges];
+
+      meta_rectangle_edge_list_to_string (window_edges, ", ", big_buffer);
+      meta_topic (META_DEBUG_EDGE_RESISTANCE,
+                  "Window edges for resistance  : %s\n", big_buffer);
+
+      meta_rectangle_edge_list_to_string (xinerama_edges, ", ", big_buffer);
+      meta_topic (META_DEBUG_EDGE_RESISTANCE,
+                  "Xinerama edges for resistance: %s\n", big_buffer);
+
+      meta_rectangle_edge_list_to_string (screen_edges, ", ", big_buffer);
+      meta_topic (META_DEBUG_EDGE_RESISTANCE,
+                  "Screen edges for resistance  : %s\n", big_buffer);
+    }
+#endif
+
   /*
    * 1st: Get the total number of each kind of edge
    */
@@ -871,12 +935,6 @@ initialize_grab_edge_resistance_data (MetaDisplay *display)
   edge_data->right_data.keyboard_buildup  = 0;
   edge_data->top_data.keyboard_buildup    = 0;
   edge_data->bottom_data.keyboard_buildup = 0;
-
-  edge_data->left_data.allow_past_screen_edge   = TRUE;
-  edge_data->right_data.allow_past_screen_edge  = TRUE;
-  edge_data->bottom_data.allow_past_screen_edge = TRUE;
-  edge_data->top_data.allow_past_screen_edge    = 
-    display->grab_anchor_root_y >= display->grab_initial_window_pos.y;
 }
 
 void
@@ -1079,6 +1137,7 @@ meta_window_edge_resistance_for_move (MetaWindow  *window,
                                       gboolean     is_keyboard_op)
 {
   MetaRectangle old_outer, proposed_outer, new_outer;
+  gboolean is_resize;
 
   if (window == window->display->grab_window &&
       window->display->grab_wireframe_active)
@@ -1097,13 +1156,15 @@ meta_window_edge_resistance_for_move (MetaWindow  *window,
   new_outer = proposed_outer;
 
   window->display->grab_last_user_action_was_snap = snap;
+  is_resize = FALSE;
   if (apply_edge_resistance_to_each_side (window->display,
                                           window,
                                           &old_outer,
                                           &new_outer,
                                           timeout_func,
                                           snap,
-                                          is_keyboard_op))
+                                          is_keyboard_op,
+                                          is_resize))
     {
       /* apply_edge_resistance_to_each_side independently applies
        * resistance to both the right and left edges of new_outer as both
@@ -1146,6 +1207,12 @@ meta_window_edge_resistance_for_move (MetaWindow  *window,
               (BOX_LEFT (*reference) - BOX_LEFT (old_outer));
       *new_y = old_y + smaller_y_change +
               (BOX_TOP (*reference) - BOX_TOP (old_outer));
+
+      meta_topic (META_DEBUG_EDGE_RESISTANCE,
+                  "outer x & y move-to coordinate changed from %d,%d to %d,%d\n",
+                  proposed_outer.x, proposed_outer.y,
+                  old_outer.x + (*new_x - old_x),
+                  old_outer.y + (*new_y - old_y));
     }
 }
 
@@ -1164,7 +1231,8 @@ meta_window_edge_resistance_for_resize (MetaWindow  *window,
                                         gboolean     is_keyboard_op)
 {
   MetaRectangle old_outer, new_outer;
-  int new_outer_width, new_outer_height;
+  int proposed_outer_width, proposed_outer_height;
+  gboolean is_resize;
 
   if (window == window->display->grab_window &&
       window->display->grab_wireframe_active)
@@ -1177,24 +1245,31 @@ meta_window_edge_resistance_for_resize (MetaWindow  *window,
     {
       meta_window_get_outer_rect (window, &old_outer);
     }
-  new_outer_width  = old_outer.width  + (*new_width  - old_width);
-  new_outer_height = old_outer.height + (*new_height - old_height);
+  proposed_outer_width  = old_outer.width  + (*new_width  - old_width);
+  proposed_outer_height = old_outer.height + (*new_height - old_height);
   meta_rectangle_resize_with_gravity (&old_outer, 
                                       &new_outer,
                                       gravity,
-                                      new_outer_width,
-                                      new_outer_height);
+                                      proposed_outer_width,
+                                      proposed_outer_height);
 
   window->display->grab_last_user_action_was_snap = snap;
+  is_resize = TRUE;
   if (apply_edge_resistance_to_each_side (window->display,
                                           window,
                                           &old_outer,
                                           &new_outer,
                                           timeout_func,
                                           snap,
-                                          is_keyboard_op))
+                                          is_keyboard_op,
+                                          is_resize))
     {
       *new_width  = old_width  + (new_outer.width  - old_outer.width);
       *new_height = old_height + (new_outer.height - old_outer.height);
+
+      meta_topic (META_DEBUG_EDGE_RESISTANCE,
+                  "outer width & height got changed from %d,%d to %d,%d\n",
+                  proposed_outer_width, proposed_outer_height,
+                  new_outer.width, new_outer.height);
     }
 }
